@@ -1,23 +1,25 @@
-use crate::SecureChannel;
+use crate::{SecureChannelNewKeyExchanger, SecureChannelVault, SecureChannelWorker};
 use async_trait::async_trait;
-use ockam_core::{Address, Message, Result, Route, Routed, TransportMessage, Worker};
-use ockam_key_exchange_xx::XXNewKeyExchanger;
+use ockam_core::{Address, Message, Result, Routed, TransportMessage, Worker};
 use ockam_node::Context;
-use ockam_vault_sync_core::VaultSync;
 use rand::random;
 use serde::{Deserialize, Serialize};
 use tracing::debug;
 
 /// SecureChannelListener listens for messages from SecureChannel initiators
 /// and creates responder SecureChannels
-pub struct SecureChannelListener {
-    vault: VaultSync,
+pub struct SecureChannelListener<V: SecureChannelVault, N: SecureChannelNewKeyExchanger> {
+    new_key_exchanger: N,
+    vault: V,
 }
 
-impl SecureChannelListener {
+impl<V: SecureChannelVault, N: SecureChannelNewKeyExchanger> SecureChannelListener<V, N> {
     /// Create a new SecureChannelListener.
-    pub fn new(vault: VaultSync) -> Self {
-        Self { vault }
+    pub fn new(new_key_exchanger: N, vault: V) -> Self {
+        Self {
+            new_key_exchanger,
+            vault,
+        }
     }
 }
 
@@ -50,7 +52,9 @@ impl CreateResponderChannelMessage {
 }
 
 #[async_trait]
-impl Worker for SecureChannelListener {
+impl<V: SecureChannelVault, N: SecureChannelNewKeyExchanger> Worker
+    for SecureChannelListener<V, N>
+{
     type Message = CreateResponderChannelMessage;
     type Context = Context;
 
@@ -70,18 +74,15 @@ impl Worker for SecureChannelListener {
             &address_local, &address_remote
         );
 
-        let new_key_exchanger = XXNewKeyExchanger::new(self.vault.start_another()?);
-
-        let channel = SecureChannel::new(
+        let channel = SecureChannelWorker::new(
             false,
             reply.clone(),
             address_remote.clone(),
             address_local.clone(),
-            msg.callback_address()
-                .as_ref()
-                .map(|a| Route::new().append(a.clone()).into()),
-            &new_key_exchanger,
-            self.vault.start_another()?,
+            msg.callback_address().clone(),
+            None,
+            self.new_key_exchanger.responder()?,
+            self.vault.clone(),
         )?;
 
         ctx.start_worker(vec![address_remote.clone(), address_local], channel)
